//
//  NANetDevice.m
//  NetAudioIF
//
//  Created by novi on 平成21/10/16.
//  Copyright 2009 novi. All rights reserved.
//

#import "NANetDevice.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <strings.h>
#include <time.h>

#import <OmniNetworking/OmniNetworking.h>
#import <OmniNetworking/ONTCPSocket.h>

NSString* keyDeviceDataPort = @"deviceDataPort";
NSString* keyDeviceCommandPort = @"deviceCommandPort";
NSString* keyDeviceIPAddress = @"deviceIP";
NSString* keyServerDataPort = @"serverDataPort";

#define kNAPacketSize 512
#define kNADataPacketSize 300

#define kNATypeOLED 1
#define kNATypeDec 2
#define kNATypeHw 3

#define kNATypeOLEDPower 3
#define kNATypeOLEDRect 1
#define kNATypeOLEDData 2

#define kNATypeDecVolume 3
#define kNATypeDecStop 1
#define kNATypeDecStart 2
#define kNATypeDecReset 4


@implementation NANetDevice

@synthesize timeout = m_timeout;
@synthesize data = m_data;

@synthesize dataSendBytes = m_dataSendBytes;

@synthesize deviceOLEDPower = m_deviceOLEDPower;
@synthesize deviceVolume = m_deviceVolume;

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	NSLog(@"%@ %@", keyPath, [object description]);
	
	if ([keyPath isEqualToString:@"deviceOLEDPower"]) {
		uint8_t d;
		if (self.deviceOLEDPower) {
			d = 1;
		} else {
			d = 0;
		}
		[self _sendCommand:kNATypeOLED subtype:kNATypeOLEDPower data:[NSData dataWithBytes:&d length:1]];
	}
	if ([keyPath isEqualToString:@"deviceVolume"]) {
		uint8_t vol[2];
		vol[0] = 2;
		vol[1] = m_deviceVolume;
		[self _sendCommand:kNATypeDec subtype:kNATypeDecVolume data:[NSData dataWithBytes:vol length:2]];
	}
}

- (id)init
{
	self = [super init];
	if (self != nil) {
		self.deviceVolume = 50;
		[self addObserver:self forKeyPath:@"deviceOLEDPower" options:NSKeyValueObservingOptionNew context:NULL];
		[self addObserver:self forKeyPath:@"deviceVolume" options:NSKeyValueObservingOptionNew context:NULL];
	}
	return self;
}

- (void)_dataSendThread:(id)sender
{
	id pool = [[NSAutoreleasePool alloc] init];
	
	m_dataSendError = NO;
	m_timeout = 0;
	m_isDataSendRunning = YES;
	m_dataSendBytes = 0;
	
	int sockfd, len;
    struct sockaddr_in address;
	
	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	
	int recvfd = socket(AF_INET, SOCK_DGRAM, 0);
	
	struct sockaddr_in saddr;	
	bzero ((char *) &saddr, sizeof (saddr));
	
	NSUserDefaults* ud = [NSUserDefaults standardUserDefaults];
	
	UInt16 srvPort = [ud integerForKey:keyServerDataPort];
	
    saddr.sin_family = AF_INET;
    saddr.sin_addr.s_addr = INADDR_ANY;
    saddr.sin_port = htons (srvPort);
	if (bind (recvfd, (struct sockaddr *) &saddr, sizeof (saddr)) < 0) {
		m_dataSendError = YES;
        return;
    }
	
	
	NSString* dstIP = [ud stringForKey:keyDeviceIPAddress];
	UInt16 dstPort = [ud integerForKey:keyDeviceDataPort];
	
    address.sin_family = AF_INET;
    address.sin_addr.s_addr = inet_addr([dstIP UTF8String]);
    address.sin_port = htons(dstPort);
    len = sizeof(address);
	
	fd_set fds, readfds;
	FD_ZERO(&readfds);
	FD_SET(recvfd, &readfds);
	
	struct timeval timeout;
	timeout.tv_sec = 2;
	timeout.tv_usec = 0;
	
	char msg[kNAPacketSize];
	NSUInteger loc = 0;
	while (1) {
		if (m_dataSendThreadAlive == NO) {
			break;
		}
		
		int readsize = 0;
		if (loc+kNAPacketSize >= [self.data length]) {
			// Last data
			[self.data getBytes:msg range:NSMakeRange(loc, [self.data length] - loc)];
			readsize = [self.data length] - loc;
		} else {
			[self.data getBytes:msg range:NSMakeRange(loc, kNAPacketSize)];
			readsize = kNAPacketSize;
		}
		loc += kNAPacketSize;
		
		m_dataSendBytes += readsize;
		sendto(sockfd, msg, readsize, 0, (struct sockaddr *)&address, sizeof (struct sockaddr));
		//printf("send\n");
		if (readsize < kNAPacketSize) {
			break;
		}
		
		
		// Wait for ACK message with timeout
		memcpy(&fds, &readfds, sizeof(fds));
		if (select(recvfd+1, &fds, NULL, NULL, &timeout) == 0) {
			printf("Time out\n");
			[self willChangeValueForKey:@"timeout"];
			m_timeout++;
			[self didChangeValueForKey:@"timeout"];
			if (m_timeout >= 10) {
				m_dataSendError = YES;
				break;
			}
			continue;
		}
		
		// If receive data is available
		if (FD_ISSET(recvfd, &fds)) {
			char buf[1024];
			int recvsize = recv(recvfd, buf, 1024, 0);
		//	printf("recv %d byte\n", recvsize);
			if (strncmp(buf, "ACK", 3) == 0) {
				continue;
			} else if (strncmp(buf, "FULL", 4) ==0) {
				sleep(1);
				continue;
			} else {
			//	printf("res : %s\n", buf);
				continue;
			}
		}
	}
	
	close(sockfd);
	close(recvfd);
	m_isDataSendRunning = NO;
	[pool drain];
}

- (void)_serverLoop:(id)sender
{
	id pool = [[NSAutoreleasePool alloc] init];
	
	NSUserDefaults* ud = [NSUserDefaults standardUserDefaults];
	
	if (! m_tcpServer) {
		m_tcpServer = (ONTCPSocket*)[ONTCPSocket tcpSocket];
	}
	[m_tcpServer abortSocket];
	[m_tcpServer startListeningOnLocalPort:[ud integerForKey:keyDeviceCommandPort] allowingAddressReuse:YES];
	[m_tcpServer acceptConnection];
	
	NSLog(@"connection accepted");
	
	while (1) {
		sleep(1);
		if (! [m_tcpServer isConnected]) {
			break;
		}
		if (m_tcpServerAlive == NO) {
			break;
		}
	}
	
	NSLog(@"disconnected");
	m_tcpServer = nil;
	[pool drain];
}

- (void)_sendCommand:(UInt8)type subtype:(UInt8)st data:(NSData*)data
{
	NSLog(@"%x %x %d byte", type, st, [data length]);
	uint8_t header[2];
	header[0] = type; header[1] = st;
	NSMutableData* hdata = [NSMutableData dataWithBytes:header length:2];
	[hdata appendData:data];
	@synchronized(m_tcpServer) {
		[m_tcpServer writeData:hdata]; 
	}
}

- (void)startDevice:(id)sender
{
	m_tcpServerAlive = NO;
	
	if (! m_tcpServer) {
		m_tcpServerAlive = YES;
		[NSThread detachNewThreadSelector:@selector(_serverLoop:) toTarget:self withObject:self];
	}
	
}
- (BOOL)dataSendStart
{
	if (! self.data) {
		return NO;
	}
	if (m_isDataSendRunning) {
		return NO;
	}
	m_dataSendThreadAlive = YES;
	[self _sendCommand:kNATypeDec subtype:kNATypeDecStart data:nil];
	[NSThread detachNewThreadSelector:@selector(_dataSendThread:) toTarget:self withObject:nil];
	
	return YES;
}

- (void)dataSendStop
{
	[self _sendCommand:kNATypeDec subtype:kNATypeDecStop data:nil];
	m_dataSendThreadAlive = NO;
}

-(void) sendOLEDData:(NSData *)data
{
	int loc = 0;
	while (1) {
		int readsize = 0;
		NSData* sendData = nil;
		if (loc+kNADataPacketSize >= [data length]) {
			// Last data
			sendData = [data subdataWithRange:NSMakeRange(loc, [data length] - loc)];
			readsize = [data length] - loc;
		} else {
			sendData = [data subdataWithRange:NSMakeRange(loc, kNADataPacketSize)];
			readsize = kNADataPacketSize;
		}
		
		[self _sendCommand:kNATypeOLED subtype:kNATypeOLEDData data:sendData];
		usleep(1000*100);
		if (readsize < kNADataPacketSize) {
			break;
		}
		 loc += kNADataPacketSize;
	}
}

@end
